////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  aecxarbitration.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "aecxarbitration.h"
#include "parametersetmanager.h"

AECXARBITRATION_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ArbitrationEndType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "endSelection") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)endSelection, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "tableID") == 0)
    {
        ParameterUtils::DoubleToString((double)tableID, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "quantityToModifyFirst") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)quantityToModifyFirst, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "manualGain") == 0)
    {
        ParameterUtils::DoubleToString((double)manualGain, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "manualTime") == 0)
    {
        ParameterUtils::DoubleToString((double)manualTime, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool ArbitrationEndType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "endSelection") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            endSelection = (EndKneePointType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "tableID") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            tableID = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "quantityToModifyFirst") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            quantityToModifyFirst = (aecxglobalelements::ExposureIncPrioType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "manualGain") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            manualGain = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "manualTime") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            manualTime = (UINT64)dblValue; 
        }
    }

    return __result;
}

bool ArbitrationEndType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // endSelection
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("endSelection", ParameterParserDataType::Enum, (double)(UINT32)endSelection, 0));
    }
    else if (PARAMETER_STRCMP(__name, "endSelection") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)endSelection, 0));
    }

    // tableID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tableID", ParameterParserDataType::Int32, (double)tableID, 0));
    }
    else if (PARAMETER_STRCMP(__name, "tableID") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)tableID, 0));
    }

    // quantityToModifyFirst
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("quantityToModifyFirst", ParameterParserDataType::Enum, (double)(UINT32)quantityToModifyFirst, 0));
    }
    else if (PARAMETER_STRCMP(__name, "quantityToModifyFirst") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)quantityToModifyFirst, 0));
    }

    // manualGain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("manualGain", ParameterParserDataType::Float, (double)manualGain));
    }
    else if (PARAMETER_STRCMP(__name, "manualGain") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)manualGain));
    }

    // manualTime
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("manualTime", ParameterParserDataType::UInt64, (double)manualTime, 0));
    }
    else if (PARAMETER_STRCMP(__name, "manualTime") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt64, (double)manualTime, 0));
    }

    return __result;
}

bool ReferenceSourceType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bUseStaticTuning") == 0)
    {
        ParameterUtils::DoubleToString((double)bUseStaticTuning, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningGain") == 0)
    {
        ParameterUtils::DoubleToString((double)staticTuningGain, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningTime") == 0)
    {
        ParameterUtils::DoubleToString((double)staticTuningTime, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "frameSelection") == 0)
    {
        __result = frameSelection.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "frameExposureTypeSelection") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)frameExposureTypeSelection, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool ReferenceSourceType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "bUseStaticTuning") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            bUseStaticTuning = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningGain") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            staticTuningGain = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningTime") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            staticTuningTime = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "frameSelection") == 0)
    {
        __result = frameSelection.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "frameExposureTypeSelection") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            frameExposureTypeSelection = (aecxglobalelements::AECXExposureSelection)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool ReferenceSourceType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // bUseStaticTuning
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bUseStaticTuning", ParameterParserDataType::Boolean, (double)bUseStaticTuning, 0));
    }
    else if (PARAMETER_STRCMP(__name, "bUseStaticTuning") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)bUseStaticTuning, 0));
    }

    // staticTuningGain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("staticTuningGain", ParameterParserDataType::Float, (double)staticTuningGain));
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningGain") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)staticTuningGain));
    }

    // staticTuningTime
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("staticTuningTime", ParameterParserDataType::UInt32, (double)staticTuningTime, 0));
    }
    else if (PARAMETER_STRCMP(__name, "staticTuningTime") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)staticTuningTime, 0));
    }

    // frameSelection
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("frameSelection", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "frameSelection") == 0)
    {
        __result = frameSelection.GetInfo(&__path[__pos], __info);
    }

    // frameExposureTypeSelection
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("frameExposureTypeSelection", ParameterParserDataType::Enum, (double)(UINT32)frameExposureTypeSelection, 0));
    }
    else if (PARAMETER_STRCMP(__name, "frameExposureTypeSelection") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)frameExposureTypeSelection, 0));
    }

    return __result;
}

bool ArbitrationReferenceType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dataID") == 0)
    {
        ParameterUtils::DoubleToString((double)dataID, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "description") == 0)
    {
        PARAMETER_STRCPY(__value, __size, description);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "bEnableOnlyForBracketedSnapshot") == 0)
    {
        ParameterUtils::DoubleToString((double)bEnableOnlyForBracketedSnapshot, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "bracketedEVType") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)bracketedEVType, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "referenceSource") == 0)
    {
        __result = referenceSource.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "endSettings") == 0)
    {
        __result = endSettings.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool ArbitrationReferenceType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "dataID") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dataID = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "description") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }
    else if (PARAMETER_STRCMP(__name, "bEnableOnlyForBracketedSnapshot") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            bEnableOnlyForBracketedSnapshot = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "bracketedEVType") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            bracketedEVType = (aecxglobalelements::EVOffsetType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "referenceSource") == 0)
    {
        __result = referenceSource.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "endSettings") == 0)
    {
        __result = endSettings.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool ArbitrationReferenceType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // dataID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dataID", ParameterParserDataType::Int32, (double)dataID, 0));
    }
    else if (PARAMETER_STRCMP(__name, "dataID") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)dataID, 0));
    }

    // description
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("description", ParameterParserDataType::String, description));
    }
    else if (PARAMETER_STRCMP(__name, "description") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, description));
    }

    // bEnableOnlyForBracketedSnapshot
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bEnableOnlyForBracketedSnapshot", ParameterParserDataType::Boolean, (double)bEnableOnlyForBracketedSnapshot, 0));
    }
    else if (PARAMETER_STRCMP(__name, "bEnableOnlyForBracketedSnapshot") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)bEnableOnlyForBracketedSnapshot, 0));
    }

    // bracketedEVType
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("bracketedEVType", ParameterParserDataType::Enum, (double)(UINT32)bracketedEVType, 0));
    }
    else if (PARAMETER_STRCMP(__name, "bracketedEVType") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)bracketedEVType, 0));
    }

    // referenceSource
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("referenceSource", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "referenceSource") == 0)
    {
        __result = referenceSource.GetInfo(&__path[__pos], __info);
    }

    // endSettings
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("endSettings", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "endSettings") == 0)
    {
        __result = endSettings.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool ISOSettingType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "previewISO100Gain") == 0)
    {
        ParameterUtils::DoubleToString((double)previewISO100Gain, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "enableISOQuantization") == 0)
    {
        ParameterUtils::DoubleToString((double)enableISOQuantization, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "quantizationSteps") == 0)
    {
        if (__index < quantizationStepsCount)
        {
            __result = quantizationSteps[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool ISOSettingType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "previewISO100Gain") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            previewISO100Gain = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableISOQuantization") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableISOQuantization = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "quantizationSteps") == 0)
    {
        if (__index < quantizationStepsCount)
        {
            __result = quantizationSteps[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool ISOSettingType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // previewISO100Gain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("previewISO100Gain", ParameterParserDataType::Float, (double)previewISO100Gain));
    }
    else if (PARAMETER_STRCMP(__name, "previewISO100Gain") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)previewISO100Gain));
    }

    // enableISOQuantization
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableISOQuantization", ParameterParserDataType::Boolean, (double)enableISOQuantization, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableISOQuantization") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableISOQuantization, 0));
    }

    // quantizationSteps
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("quantizationSteps", ParameterParserDataType::Structure, true, false, quantizationStepsCount));
    }
    else if (PARAMETER_STRCMP(__name, "quantizationSteps") == 0)
    {
        if (__index < quantizationStepsCount)
        {
            __result = quantizationSteps[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, quantizationStepsCount));
        }
    }

    return __result;
}

bool EVIncreaseCapType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableEVGainCapping") == 0)
    {
        ParameterUtils::DoubleToString((double)enableEVGainCapping, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "maxEVCappingGain") == 0)
    {
        ParameterUtils::DoubleToString((double)maxEVCappingGain, __value, __size);
        __result = true;
    }

    return __result;
}

bool EVIncreaseCapType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableEVGainCapping") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableEVGainCapping = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "maxEVCappingGain") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            maxEVCappingGain = (float)dblValue; 
        }
    }

    return __result;
}

bool EVIncreaseCapType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableEVGainCapping
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableEVGainCapping", ParameterParserDataType::Boolean, (double)enableEVGainCapping, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableEVGainCapping") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableEVGainCapping, 0));
    }

    // maxEVCappingGain
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("maxEVCappingGain", ParameterParserDataType::Float, (double)maxEVCappingGain));
    }
    else if (PARAMETER_STRCMP(__name, "maxEVCappingGain") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)maxEVCappingGain));
    }

    return __result;
}

bool EVIncreaseType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableEVTableIncrease") == 0)
    {
        ParameterUtils::DoubleToString((double)enableEVTableIncrease, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "increaseCap") == 0)
    {
        __result = increaseCap.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool EVIncreaseType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enableEVTableIncrease") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableEVTableIncrease = (BOOL)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "increaseCap") == 0)
    {
        __result = increaseCap.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool EVIncreaseType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enableEVTableIncrease
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableEVTableIncrease", ParameterParserDataType::Boolean, (double)enableEVTableIncrease, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableEVTableIncrease") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableEVTableIncrease, 0));
    }

    // increaseCap
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("increaseCap", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "increaseCap") == 0)
    {
        __result = increaseCap.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool EVArbitrationType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "EVTableIncrease") == 0)
    {
        __result = EVTableIncrease.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "enableEVTableDecrease") == 0)
    {
        ParameterUtils::DoubleToString((double)enableEVTableDecrease, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool EVArbitrationType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "EVTableIncrease") == 0)
    {
        __result = EVTableIncrease.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "enableEVTableDecrease") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableEVTableDecrease = (BOOL)dblValue; 
        }
    }

    return __result;
}

bool EVArbitrationType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // EVTableIncrease
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("EVTableIncrease", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "EVTableIncrease") == 0)
    {
        __result = EVTableIncrease.GetInfo(&__path[__pos], __info);
    }

    // enableEVTableDecrease
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableEVTableDecrease", ParameterParserDataType::Boolean, (double)enableEVTableDecrease, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableEVTableDecrease") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableEVTableDecrease, 0));
    }

    return __result;
}

bool AECXArbitrationType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)moduleID, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "EVArbitration") == 0)
    {
        __result = EVArbitration.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "ISO") == 0)
    {
        __result = ISO.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "referenceArray") == 0)
    {
        if (__index < referenceArrayCount)
        {
            __result = referenceArray[__index].GetValue(&__path[__pos], __value, __size);
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableMFHDRLowLightSmartFPS") == 0)
    {
        ParameterUtils::DoubleToString((double)enableMFHDRLowLightSmartFPS, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool AECXArbitrationType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            moduleID = (aecxglobalelements::AECXModuleIDType)(UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "EVArbitration") == 0)
    {
        __result = EVArbitration.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "ISO") == 0)
    {
        __result = ISO.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "referenceArray") == 0)
    {
        if (__index < referenceArrayCount)
        {
            __result = referenceArray[__index].SetValue(&__path[__pos], __value);
        }
    }
    else if (PARAMETER_STRCMP(__name, "enableMFHDRLowLightSmartFPS") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            enableMFHDRLowLightSmartFPS = (BOOL)dblValue; 
        }
    }

    return __result;
}

bool AECXArbitrationType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // moduleID
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("moduleID", ParameterParserDataType::Enum, (double)(UINT32)moduleID, 0));
    }
    else if (PARAMETER_STRCMP(__name, "moduleID") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)moduleID, 0));
    }

    // EVArbitration
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("EVArbitration", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "EVArbitration") == 0)
    {
        __result = EVArbitration.GetInfo(&__path[__pos], __info);
    }

    // ISO
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ISO", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "ISO") == 0)
    {
        __result = ISO.GetInfo(&__path[__pos], __info);
    }

    // referenceArray
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("referenceArray", ParameterParserDataType::Structure, true, false, referenceArrayCount));
    }
    else if (PARAMETER_STRCMP(__name, "referenceArray") == 0)
    {
        if (__index < referenceArrayCount)
        {
            __result = referenceArray[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, referenceArrayCount));
        }
    }

    // enableMFHDRLowLightSmartFPS
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enableMFHDRLowLightSmartFPS", ParameterParserDataType::Boolean, (double)enableMFHDRLowLightSmartFPS, 0));
    }
    else if (PARAMETER_STRCMP(__name, "enableMFHDRLowLightSmartFPS") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Boolean, (double)enableMFHDRLowLightSmartFPS, 0));
    }

    return __result;
}

bool AECXArbitrationType::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool AECXArbitrationType::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool AECXArbitrationType::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool AECXArbitrationType::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool AECXArbitrationType::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool AECXArbitrationType::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool AntibandingType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ATB50Index") == 0)
    {
        ParameterUtils::DoubleToString((double)ATB50Index, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ATB60Index") == 0)
    {
        ParameterUtils::DoubleToString((double)ATB60Index, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool AntibandingType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "ATB50Index") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ATB50Index = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ATB60Index") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ATB60Index = (INT32)dblValue; 
        }
    }

    return __result;
}

bool AntibandingType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // ATB50Index
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ATB50Index", ParameterParserDataType::Int32, (double)ATB50Index, 0));
    }
    else if (PARAMETER_STRCMP(__name, "ATB50Index") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)ATB50Index, 0));
    }

    // ATB60Index
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ATB60Index", ParameterParserDataType::Int32, (double)ATB60Index, 0));
    }
    else if (PARAMETER_STRCMP(__name, "ATB60Index") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)ATB60Index, 0));
    }

    return __result;
}

bool AdjustDRCType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "safeTimeMaxRatio") == 0)
    {
        ParameterUtils::DoubleToString((double)safeTimeMaxRatio, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "safeGainMaxRatio") == 0)
    {
        ParameterUtils::DoubleToString((double)safeGainMaxRatio, __value, __size);
        __result = true;
    }

    return __result;
}

bool AdjustDRCType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "safeTimeMaxRatio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            safeTimeMaxRatio = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "safeGainMaxRatio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            safeGainMaxRatio = (float)dblValue; 
        }
    }

    return __result;
}

bool AdjustDRCType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // safeTimeMaxRatio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("safeTimeMaxRatio", ParameterParserDataType::Float, (double)safeTimeMaxRatio));
    }
    else if (PARAMETER_STRCMP(__name, "safeTimeMaxRatio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)safeTimeMaxRatio));
    }

    // safeGainMaxRatio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("safeGainMaxRatio", ParameterParserDataType::Float, (double)safeGainMaxRatio));
    }
    else if (PARAMETER_STRCMP(__name, "safeGainMaxRatio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)safeGainMaxRatio));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReadEnum_EndKneePointType(
    ParameterFileSymbolTableEntry* const entry,
    EndKneePointType& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(EndKneePointType::EndFirstTableKneePoint):
            enumValue = EndKneePointType::EndFirstTableKneePoint;
            result    = true;
            break;
        case static_cast<UINT32>(EndKneePointType::EndLastTableKneePoint):
            enumValue = EndKneePointType::EndLastTableKneePoint;
            result    = true;
            break;
        case static_cast<UINT32>(EndKneePointType::EndManualSettings):
            enumValue = EndKneePointType::EndManualSettings;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to EndKneePointType", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
AECXArbitrationTypeClass::AECXArbitrationTypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(EVArbitration);
    PARAMETER_INITIALIZE(ISO);
    PARAMETER_INITIALIZE(referenceArrayCount);
    PARAMETER_INITIALIZE(referenceArray);
    PARAMETER_INITIALIZE(enableMFHDRLowLightSmartFPS);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
AECXArbitrationTypeClass::AECXArbitrationTypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(EVArbitration);
    PARAMETER_INITIALIZE(ISO);
    PARAMETER_INITIALIZE(referenceArray);
    PARAMETER_INITIALIZE(enableMFHDRLowLightSmartFPS);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
AECXArbitrationTypeClass::~AECXArbitrationTypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* AECXArbitrationTypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "aecxarbitration")) == 0)
    {
        name = "aecxarbitration";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load ArbitrationEndType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ArbitrationEndType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(endSelection);
    PARAMETER_INITIALIZE(tableID);
    PARAMETER_INITIALIZE(quantityToModifyFirst);
    PARAMETER_INITIALIZE(manualGain);
    PARAMETER_INITIALIZE(manualTime);

    result = aecxarbitration::ReadEnum_EndKneePointType(entry, endSelection, alignment);
    SYMBOL_TABLE_ENTRY_READ(entry, result, tableID, alignment)
    if (result)
    {
        result = aecxglobalelements::ReadEnum_ExposureIncPrioType(entry, quantityToModifyFirst, alignment);
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, manualGain, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, manualTime, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load ReferenceSourceType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ReferenceSourceType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(bUseStaticTuning);
    PARAMETER_INITIALIZE(staticTuningGain);
    PARAMETER_INITIALIZE(staticTuningTime);
    PARAMETER_INITIALIZE(frameSelection);
    PARAMETER_INITIALIZE(frameExposureTypeSelection);

    SYMBOL_TABLE_ENTRY_READ(entry, result, bUseStaticTuning, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, staticTuningGain, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, staticTuningTime, alignment)
    if (result)
    {
        result = frameSelection.Load(entry, alignment);
    }
    if (result)
    {
        result = aecxglobalelements::ReadEnum_AECXExposureSelection(entry, frameExposureTypeSelection, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load ArbitrationReferenceType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ArbitrationReferenceType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(dataID);
    PARAMETER_INITIALIZE(description);
    PARAMETER_INITIALIZE(bEnableOnlyForBracketedSnapshot);
    PARAMETER_INITIALIZE(bracketedEVType);
    PARAMETER_INITIALIZE(referenceSource);
    PARAMETER_INITIALIZE(endSettings);

    SYMBOL_TABLE_ENTRY_READ(entry, result, dataID, alignment)

    // Reading String description from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, description, count, alignment)
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, bEnableOnlyForBracketedSnapshot, alignment)
    if (result)
    {
        result = aecxglobalelements::ReadEnum_EVOffsetType(entry, bracketedEVType, alignment);
    }
    if (result)
    {
        result = referenceSource.Load(entry, alignment);
    }
    if (result)
    {
        result = endSettings.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load ISOSettingType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool ISOSettingType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(previewISO100Gain);
    PARAMETER_INITIALIZE(enableISOQuantization);
    PARAMETER_INITIALIZE(quantizationSteps);

    SYMBOL_TABLE_ENTRY_READ(entry, result, previewISO100Gain, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableISOQuantization, alignment)

    // Reading Array quantizationSteps from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        quantizationStepsCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, quantizationStepsCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            quantizationStepsID = pointerEntry->ID;
            DupObjInfo* quantizationStepsDupInfo = pointerEntry->GetDupObject(quantizationStepsCount, file, line);
            if (quantizationStepsDupInfo != nullptr && quantizationStepsDupInfo->Object != nullptr)
            {
                quantizationSteps = (aecxglobalelements::TriggerSystemEndLevelType*)quantizationStepsDupInfo->Object;
                if (quantizationSteps == nullptr)
                {
                    if (quantizationStepsDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "quantizationSteps duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            quantizationStepsDupInfo->File, quantizationStepsDupInfo->Line, quantizationStepsDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "quantizationSteps duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            quantizationStepsDupInfo->Line, quantizationStepsDupInfo->RefCount);
                    }
                }
            }

            if (quantizationSteps == nullptr)
            {
                if (result && quantizationStepsCount > 0)
                {
                    quantizationSteps = PARAMETER_NEW aecxglobalelements::TriggerSystemEndLevelType[quantizationStepsCount];
                    (void)pointerEntry->SetDupObject(quantizationSteps, quantizationStepsCount, file, line);
                }
                if (!result || quantizationSteps == nullptr)
                {
                    if (quantizationStepsCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate quantizationSteps\n");
                    }
                    quantizationStepsCount = 0;
                }
                for (UINT32 i = 0; result && i < quantizationStepsCount; i++)
                {
                    result = quantizationSteps[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (quantizationSteps == nullptr)
        {
            quantizationStepsCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load EVIncreaseCapType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool EVIncreaseCapType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableEVGainCapping);
    PARAMETER_INITIALIZE(maxEVCappingGain);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableEVGainCapping, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, maxEVCappingGain, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load EVIncreaseType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool EVIncreaseType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enableEVTableIncrease);
    PARAMETER_INITIALIZE(increaseCap);

    SYMBOL_TABLE_ENTRY_READ(entry, result, enableEVTableIncrease, alignment)
    if (result)
    {
        result = increaseCap.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load EVArbitrationType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool EVArbitrationType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(EVTableIncrease);
    PARAMETER_INITIALIZE(enableEVTableDecrease);

    result = EVTableIncrease.Load(entry, alignment);
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableEVTableDecrease, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load AECXArbitrationType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AECXArbitrationType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(moduleID);
    PARAMETER_INITIALIZE(EVArbitration);
    PARAMETER_INITIALIZE(ISO);
    PARAMETER_INITIALIZE(referenceArray);
    PARAMETER_INITIALIZE(enableMFHDRLowLightSmartFPS);

    result = module_version.Load(entry, alignment);
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = aecxglobalelements::ReadEnum_AECXModuleIDType(entry, moduleID, alignment);
    }
    if (result)
    {
        result = EVArbitration.Load(entry, alignment);
    }
    if (result)
    {
        result = ISO.Load(entry, alignment);
    }

    // Reading Array referenceArray from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        referenceArrayCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, referenceArrayCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            referenceArrayID = pointerEntry->ID;
            DupObjInfo* referenceArrayDupInfo = pointerEntry->GetDupObject(referenceArrayCount, file, line);
            if (referenceArrayDupInfo != nullptr && referenceArrayDupInfo->Object != nullptr)
            {
                referenceArray = (ArbitrationReferenceType*)referenceArrayDupInfo->Object;
                if (referenceArray == nullptr)
                {
                    if (referenceArrayDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "referenceArray duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            referenceArrayDupInfo->File, referenceArrayDupInfo->Line, referenceArrayDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "referenceArray duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            referenceArrayDupInfo->Line, referenceArrayDupInfo->RefCount);
                    }
                }
            }

            if (referenceArray == nullptr)
            {
                if (result && referenceArrayCount > 0)
                {
                    referenceArray = PARAMETER_NEW ArbitrationReferenceType[referenceArrayCount];
                    (void)pointerEntry->SetDupObject(referenceArray, referenceArrayCount, file, line);
                }
                if (!result || referenceArray == nullptr)
                {
                    if (referenceArrayCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate referenceArray\n");
                    }
                    referenceArrayCount = 0;
                }
                for (UINT32 i = 0; result && i < referenceArrayCount; i++)
                {
                    result = referenceArray[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (referenceArray == nullptr)
        {
            referenceArrayCount = 0;
        }
    }
    SYMBOL_TABLE_ENTRY_READ(entry, result, enableMFHDRLowLightSmartFPS, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AECXArbitrationType::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AECXArbitrationType::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load AntibandingType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AntibandingType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(ATB50Index);
    PARAMETER_INITIALIZE(ATB60Index);

    SYMBOL_TABLE_ENTRY_READ(entry, result, ATB50Index, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, ATB60Index, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load AdjustDRCType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AdjustDRCType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(safeTimeMaxRatio);
    PARAMETER_INITIALIZE(safeGainMaxRatio);

    SYMBOL_TABLE_ENTRY_READ(entry, result, safeTimeMaxRatio, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, safeGainMaxRatio, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload ArbitrationReferenceType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ArbitrationReferenceType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(description))
    {
        PARAMETER_DELETE[] description;
    }
    description = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload ISOSettingType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ISOSettingType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(quantizationSteps))
    {
        PARAMETER_DELETE[] quantizationSteps;
    }
    quantizationSteps = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload AECXArbitrationType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AECXArbitrationType::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    ISO.Unload(__table);
    for (UINT32 i = 0; i < referenceArrayCount; i++)
    {
        referenceArray[i].Unload(__table);
    }
    if (__table != nullptr && __table->ReleaseDupObject(referenceArray))
    {
        PARAMETER_DELETE[] referenceArray;
    }
    referenceArray = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AECXArbitrationType::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AECXArbitrationType::Release()
{
    ((AECXArbitrationTypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 AECXArbitrationType::GetModified()
{
    return ((AECXArbitrationTypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* AECXArbitrationTypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    AECXArbitrationTypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (AECXArbitrationTypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW AECXArbitrationTypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create AECXArbitrationTypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create AECXArbitrationTypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool AECXArbitrationTypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        AECXArbitrationTypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "AECXArbitrationTypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AECXArbitrationTypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void AECXArbitrationTypeClass::Release()
{
    ParameterModule::Release();
}

AECXARBITRATION_NAMESPACE_END
